/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

package com.sunlabs.archsim.fleet.ships.tokenaccumulator;
import com.sunlabs.archsim.base.simulation.Delay;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.OutputFunction;
import com.sunlabs.archsim.gasp.Terminal;
import com.sunlabs.archsim.gasp.Action;
import com.sunlabs.archsim.gasp.terminals.InputTerminal;
import com.sunlabs.archsim.gasp.terminals.OutputTerminal;
import com.sunlabs.archsim.globals.Globals;

class PassOnAction extends Action {


	/* */
	private OutputTerminal out;
	private InputTerminal in;
        
        public PassOnAction() {
            this("PassOnAction");
        }
        
	public PassOnAction(String name){
		super(name);

	}
	
	private InputTerminal attachIn(Terminal t, InputTerminal localTerminal) {
            if (localTerminal == null) {
                    if (t instanceof InputTerminal) {
                            t.attachAction(this);
                            numInputs++;
                            terminals.add(t);            
                            return (InputTerminal)t;
                    } else {
                            // the terminal was of a wrong type
                            director.fatalError("OutputAction, " + getName() + ", " + getID() +  
                                                ": Terminal " + t.getName() + " is of a bad type");
                            return null;
                    }
            } else {
                    // this was the second addition of the same name
                    director.fatalError("OutputAction, " + getName() + ", " + getID() 
					+ ": Terminal " + t.getName() + " cannot be attached twice");
                    return null;
            }
        }

        private OutputTerminal attachOut(Terminal t, OutputTerminal localTerminal) {
            if (localTerminal == null) {
                    if (t instanceof OutputTerminal) {
                            t.attachAction(this);
                            terminals.add(t);            
                            return (OutputTerminal)t;
                    } else {
                            // the terminal was of a wrong type
                            director.fatalError("OutputAction, " + getName() + ", " + getID() +  
                                                ": Terminal " + t.getName() + " is of a bad type");
                            return null;
                    }
            } else {
                    // this was the second addition of the same name
                    director.fatalError("OutputAction, " + getName() + ", " + getID() 
					+ ": Terminal " + t.getName() + " cannot be attached twice");
                    return null;
            }
        }

	/**
	 * Attach a terminal.
	 * Attach only terminals that are needed for this action, ignore the rest.
	 * Self check ensures that all needed terminals are attached
	 */
	public void attach(Terminal t) {
                if (t != null) {
                        if (t instanceof OutputTerminal) {
                                out= attachOut(t, out); 
                                // assign output function with the terminal
                                outputFunctions.put(t.getBaseName(), new PassOnFunction("PassOnFunction"));
                                director.journal("Adding output " + t.getBaseName() + " to " + getName());
                                if (out == null) {
                                        director.error("out == null in " + getName());
                                }
                        } else if (t instanceof InputTerminal) {
                                in= attachIn(t, in); 
                                director.journal("Adding input " + t.getBaseName() + " to " + getName());
                                if (in == null) {
                                        director.error("in == null in " + getName());
                                }
                        } else {

                                // unknown terminal
                                director.error("Terminal " 
                                                + t.getName() 
                                                + " cannot be attached to PassOnAction "
                                                + getName() + ", id= " + getID()
                                                + "; terminal kind not recognized");
                        }
                } else {
                                 director.error("Trying to attach a null terminal to PassOnAction "
                                                + getName() + ", id= " + getID() );
                   
                }
			
		
	} // attach
	
	public InputTerminal getIn() {
            return in;
        }
        
	/** self check - are all the terminals attached, run the self check from superclass */
	public boolean selfCheck() {
		boolean OK= true;
		boolean check;
                
                check= (in != null);
		OK= OK && check;
		if (!check) {
			director.error("Action " + getAlias() + ", id= " + getID()
					+ ", does not have attached terminal in");
		}

                check= (out != null);
		OK= OK && check;
		if (!check) {
			director.error("Action " + getAlias() + ", id= " + getID()
					+ ", does not have attached terminal out");
		}
		
		// also run the self check on the superclass
		check= super.selfCheck();
		OK= OK && check;
                		
		return OK;
	} // selfCheck

        private class PassOnFunction extends OutputFunction {

                /** Global settings */
                protected Globals globals= Globals.getInstance();

                public PassOnFunction(String name) {
                        super(name);
                        Integer fDelay= globals.intValue(TokenAccumulatorDefaults.ACCUMULATOR_OUT_DELAY);
                        functionDelay= (fDelay != null)
                                        ? new Delay(fDelay.intValue())
                                        : TokenAccumulatorDefaults.ACCUMULATOR_OUT_DELAY_DEF;
                }


                public Data value() {
                        // pass on the data from the internal data in input
                        return (Data)PassOnAction.this.getIn().getData();

                } // value

        } // OutputFunction
        
} // class PassOnAction
